export type EntityUpdateFn = () => void;
export type EntityUpdateResult<T extends AnyEntity = AnyEntity> = {
    beforeProps?: GetEntityProps<T>;
    afterProps?: GetEntityProps<T>;
};
export declare class EntityUpdater<T extends AnyEntity = AnyEntity> {
    entity: T;
    updateFn: EntityUpdateFn;
    result: EntityUpdateResult<T>;
    executed: boolean;
    constructor(entity: T, updateFn: EntityUpdateFn);
    update(): EntityUpdateResult<T> | undefined;
}
export declare abstract class EntityBase<P> {
    private readonly _type;
    private readonly _id;
    private _marked;
    private _updaters;
    protected _props: P;
    constructor(type: string, id: string, props: P);
    static isEntity(obj: any): obj is AnyEntity;
    abstract validateProps(props: P): void;
    get type(): string;
    get id(): string;
    get updaters(): EntityUpdater<AnyEntity>[];
    private setProps;
    hasId(id: string): boolean;
    protected mark(): void;
    protected unmark(): void;
    protected update(updateFn: EntityUpdateFn): EntityUpdateResult<this> | undefined;
    hasChanged(): boolean;
    equalsType(entity: AnyEntity): boolean;
    equals(entity: AnyEntity): boolean;
    getProps(): P;
}
export type AnyEntity = EntityBase<any>;
export type GetEntityProps<T extends AnyEntity> = T extends EntityBase<infer Props> ? Props : never;
export type EntityBaseConstructorParamsWithProps<Props> = ConstructorParameters<typeof EntityBase<Props>>;
export type EntityBaseConstructorParams<T extends AnyEntity> = EntityBaseConstructorParamsWithProps<GetEntityProps<T>>;
