"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityBase = exports.EntityUpdater = void 0;
const lodash_1 = __importDefault(require("lodash"));
class EntityUpdater {
    constructor(entity, updateFn) {
        this.entity = entity;
        this.updateFn = updateFn;
        this.result = {};
        this.executed = false;
    }
    update() {
        if (this.executed)
            return;
        this.result.beforeProps = this.entity.getProps();
        this.updateFn();
        this.result.afterProps = this.entity.getProps();
        this.executed = true;
        return this.result;
    }
}
exports.EntityUpdater = EntityUpdater;
class EntityBase {
    constructor(type, id, props) {
        this._type = type;
        this._id = id;
        this._marked = false;
        this._updaters = [];
        this.setProps(props);
    }
    static isEntity(obj) {
        return obj instanceof EntityBase;
    }
    get type() {
        return this._type;
    }
    get id() {
        return this._id;
    }
    get updaters() {
        return this._updaters;
    }
    setProps(props) {
        this.validateProps(props);
        this._props = props;
    }
    hasId(id) {
        return this.id === id;
    }
    mark() {
        this._marked = true;
    }
    unmark() {
        this._marked = false;
    }
    update(updateFn) {
        const updater = new EntityUpdater(this, updateFn);
        const result = updater.update();
        this.validateProps(this._props);
        this._updaters.push(updater);
        this.mark();
        return result;
    }
    hasChanged() {
        return this._marked;
    }
    equalsType(entity) {
        return entity instanceof this.constructor;
    }
    equals(entity) {
        if (!this.equalsType(entity))
            return false;
        return this.hasId(entity.id);
    }
    getProps() {
        return lodash_1.default.cloneDeep(this._props);
    }
}
exports.EntityBase = EntityBase;
