export declare abstract class ValueObjectBase<P> {
    private readonly _type;
    protected readonly _props: Readonly<P>;
    constructor(type: string, props: P);
    static isValueObject(obj: any): boolean;
    abstract validateProps(props: P): void;
    equalsType(obj: ValueObjectBase<P>): boolean;
    equals(obj: ValueObjectBase<P>): boolean;
    cloneWith(props: Partial<P>): this;
    clone(): this;
    get type(): string;
    get props(): Readonly<P>;
}
export type AnyValueObject = ValueObjectBase<any>;
export type GetValueObjectProps<T extends AnyValueObject> = T extends ValueObjectBase<infer Props> ? Props : any;
export type ValueObjectBaseConstructorParamsWithProps<Props> = ConstructorParameters<typeof ValueObjectBase<Props>>;
export type ValueObjectBaseConstructorParams<T extends AnyValueObject> = ValueObjectBaseConstructorParamsWithProps<GetValueObjectProps<T>>;
