"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValueObjectBase = void 0;
const lodash_1 = __importDefault(require("lodash"));
class ValueObjectBase {
    constructor(type, props) {
        this.validateProps(props);
        this._type = type;
        this._props = props;
    }
    static isValueObject(obj) {
        return obj instanceof ValueObjectBase;
    }
    equalsType(obj) {
        // return obj instanceof this.constructor;
        return this.type === obj.type;
    }
    equals(obj) {
        if (obj === null || obj === undefined)
            return false;
        if (!this.equalsType(obj))
            return false;
        return JSON.stringify(this.props) === JSON.stringify(obj.props);
    }
    cloneWith(props) {
        const clonedProps = lodash_1.default.cloneDeep(this.props);
        return new this.constructor(lodash_1.default.merge(clonedProps, props));
    }
    clone() {
        return this.cloneWith({});
    }
    get type() {
        return this._type;
    }
    get props() {
        return this._props;
    }
}
exports.ValueObjectBase = ValueObjectBase;
